<?php
# Various boxes.
#
# Copyright (C) 1999-2000 The SourceForge Crew
# Copyright (C) 2000-2006 Mathieu Roy <yeupou--gnu.org>
# Copyright (C) 2017, 2020, 2023 Ineiev
#
# This file is part of Savane.
#
# Savane is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Savane is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

function show_altrow ($i)
{
  return '<div class="' . utils_altrow ($i) . '"><span class="smaller">';
}

function show_features_boxes ()
{
  global $HTML;
  $return = '';

  # General stats.
  $return .= $HTML->box_top (
    utils_link (
      $GLOBALS['sys_home'] . "stats/",
      # TRANSLATORS: the argument is site name (like Savannah).
      sprintf(_("%s Statistics"), $GLOBALS['sys_name']), "sortbutton"
    )
  );
  $return .= show_sitestats ();
  $return .= $HTML->box_bottom ();

  # Job offers stats.
  $jobs = people_show_category_list ();

  if ($jobs)
    {
      $return .= "<br />\n";
      $return .= $HTML->box_top (_("Help Wanted"),'',1);
      $return .= $jobs;
      $return .= $HTML->box_bottom (1);
    }

  # Popular items.
  $votes = show_votes ();

  if ($votes)
    {
      $return .= "<br />\n";
      $return .= $HTML->box_top (_("Most Popular Items"), '', 1);
      $return .= $votes;
      $return .= $HTML->box_bottom (1);
    }

  # Group type stats.
  $result = db_query ("SELECT type_id, name FROM group_type ORDER BY name");
  # Try to find out how many latest registered groups per group type
  # we should print, depending on how many groups we have.
  $limit = 10;
  $count = db_numrows ($result);
  if ($count > 25)
    $limit = 2;
  elseif ($count < 2)
    $limit = 25;
  else
    $limit = round (35 / $count);

  while ($eachtype = db_fetch_array ($result))
    {
      $groupdata = show_newest_projects ($eachtype['type_id'], $limit);
      if (!$groupdata)
        continue;

      $return .= "<br />\n";
      # TRANSLATORS: the argument is project type (like Official GNU software).
      $return .= $HTML->box_top (
        sprintf (_("Newest %s Projects"), $eachtype['name']), '', 1
      );
      $return .= $groupdata;
      global $j, $sys_home;
      $return .= show_altrow ($j) . '<a href="'
        . "{$sys_home}search/?type_of_search=soft&amp;words=%%%&amp;type="
        . $eachtype['type_id'] . '">[';
      # TRANSLATORS: the argument is project type (like Official GNU software).
      $return .= sprintf ( _("all %s projects"), $eachtype['name']);
      $return .= "]</a></span></div>\n";
      $return .= $HTML->box_bottom (1);
    }
  return $return;
}

function show_sitestats ()
{
  global $sys_home;

  $return = '<span class="smaller">';
  $users = stats_getusers ();
  $return .= sprintf (
    ngettext ("%s registered user", "%s registered users", $users),
    "<strong>$users</strong>"
  );
  $return .= "</span></div>\n";
  $i = 0;
  $return .= show_altrow ($i++);
  $projects = stats_getprojects_active ();
  $return .= sprintf (
    ngettext ("%s hosted project", "%s hosted projects", $projects),
    "<strong>$projects</strong>"
  );
  $return .= "</span></div>\n";
  $result = db_query("SELECT type_id, name FROM group_type ORDER BY name");
  while ($eachtype = db_fetch_array ($result))
    {
      $return .= show_altrow ($i++);
      $return .= "&nbsp;&nbsp;- <a href=\"{$sys_home}search/"
        . '?type_of_search=soft&amp;words=%%%&amp;type='
        . $eachtype['type_id'] . '" class="center">';
      $return .= stats_getprojects_bytype_active ($eachtype['type_id']);
      $return .= ' ' . $eachtype['name'] . '</a>';
      $return .= "</span></div>\n";
    }
  $pending = stats_getprojects_pending ();
  $return .= show_altrow ($i++) . '&nbsp;&nbsp;';
  $return .= sprintf (
    ngettext (
     "+ %s registration pending", "+ %s registrations pending", $pending
    ),
    $pending
  );
  return $return . '</span>';
}

# Show projects that were added less than 2 months ago.
function show_newest_projects ($group_type, $limit)
{
  global $j, $sys_home;

  $since = time () - 2 * 30 * 24 * 3600;
  $res_newproj = db_execute ("
    SELECT group_id, unix_group_name, group_name, register_time FROM groups
    WHERE is_public = 1 AND status = 'A' AND type = ? AND register_time >= ?
    ORDER BY register_time DESC LIMIT ?", [$group_type, $since, $limit]
  );
  if (!db_numrows ($res_newproj))
    return false;

  $base_url = '';
  $res_newproj_type = db_execute (
    "SELECT type_id, base_host FROM group_type WHERE type_id = ?",
    [$group_type]
  );
  $row_newproj_type = db_fetch_array ($res_newproj_type);
  if ($row_newproj_type['base_host'])
    $base_url = 'http' . (session_issecure () ? 's' : '') . '://'
      . $row_newproj_type['base_host'];

  $return = '';
  while ($row = db_fetch_array ($res_newproj))
    if ($row['register_time'])
      {
        $return .= show_altrow ($j++) . '&nbsp;&nbsp;- <a href="'
          . "$base_url{$sys_home}projects/$row[unix_group_name]/\">"
          . $row['group_name'] . '</a>, '
          . utils_format_date ($row['register_time'], 'minimal')
          . '</span></div>';
      }
  return $return;
}

# Find out interesting bugs and return a string listing them.
# Closed and private items are ignored.
function show_votes ($limit = 10)
{
  global $sys_home;
  $vote = $summary = [];

  foreach (["bugs", "task", "support", "patch"] as $tracker)
    {
      $result = db_execute ("
        SELECT bug_id, group_id, summary,vote
        FROM $tracker WHERE vote >= 35 AND privacy = 1 AND status_id = 1
        ORDER BY vote DESC LIMIT ?", [$limit]
      );
      $rows = db_numrows ($result);
      if (!$result || $rows <= 0)
        continue;
      for ($j = 0; $j < $rows; $j++)
        {
          $item = "$tracker#" . db_result ($result, $j, 'bug_id');
          $vote[$item] = db_result ($result, $j, 'vote');
          $summary[$item] = db_result ($result, $j, 'summary');
        }
    }
  if (empty ($vote))
    return false;

  # Sort items in rank and print the first $limit ones or less.
  $return = '';
  $count = 0;
  arsort ($vote);
  foreach ($vote as $item => $vote)
    {
      $count++;
      if ($count > $limit)
        break;

      list ($tracker, $item_id) = explode ("#", $item);
      $prefix = utils_get_tracker_prefix ($tracker);

      # If the summar item is large (>30), only show the first
      # 30 characters of the story.
      if (strlen ($summary[$item]) > 30)
        {
          $summary[$item] = substr ($summary[$item], 0, 30);
          $summary[$item] = substr (
            $summary[$item], 0, strrpos ($summary[$item], ' ')
          );
          $summary[$item] .= "...";
        }
      $url = "$sys_home$tracker/?$item_id";
      $return .= show_altrow ($count) . '&nbsp;&nbsp;- '
        . "<a href=\"$url\">$prefix #$item_id</a>: &nbsp;"
        . "<a href=\"$url\">{$summary[$item]}</a>,"
        . '&nbsp;' . sprintf (ngettext ("%s vote", "%s votes", $vote), $vote)
        . '</span></div>';
    }
  return $return;
}
?>
